﻿Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.ComponentModel
Imports System.Data.Entity

Namespace ViewModel
    Public Class UrgencyListVm
        Inherits ViewModelBase

        Private ReadOnly _context As UrgentOrderContext

        Public Sub New()
            If Not IsInDesignMode Then
                _context = New UrgentOrderContext
                _context.Database.CreateIfNotExists()
                If Not _context.UrgentOrderSet.Any Then
                    Dim answere As New Answer() With {.AnswerText = "Antworttext", .Contact = "Neue Kontakt", .RequestDate = Now}
                    Dim answereList As New List(Of Answer) : answereList.Add(answere)
                    _context.UrgentOrderSet.Add(New UrgentOrder() With {.ConfirmationDate = Now, .OrderArticle = 12345, .OrderDescription = "Testartikel", .Remarks = "Beschreibung" _
                                               , .Answers = answereList, .OrderNbr = 521145, .OrderQuantity = 235, .WorkCenter = "Workcenter 123"})

                    _context.SaveChanges()
                End If



                UrgentOrderList = (From u As UrgentOrder In _context.UrgentOrderSet.Include(Function(u) u.Answers)).ToList
                'Include sorgt dafür das benötigte Daten (Answers) gleich mitgeladen werden anstatt X Befehle an die DB zu senden was viel länger dauert
                'Um in Include keinen String sondern einen LINQ mitgeben zu können ist ein Import auf System.Data.Entity notwendig

                UrgentOrdersView = CollectionViewSource.GetDefaultView(UrgentOrderList)
                AddHandler UrgentOrdersView.CollectionChanged, AddressOf ListView_Changed
                AddHandler UrgentOrdersView.CurrentChanged, AddressOf ListView_CurrentChanged

                UrgentOrdersView.MoveCurrentToFirst() 'Ersten Datensatz mal markieren

            End If
        End Sub

        Private Sub AnswersView_Changed(sender As Object, e As NotifyCollectionChangedEventArgs)
            Select Case e.Action
                Case NotifyCollectionChangedAction.Add
                    If GetType(Answer) Is e.NewItems(0).GetType Then
                        Dim tempObj = CType(e.NewItems(0), Answer)
                        'Dem UrgendOrderViewItem (Typ UrgentOrder) eine neue Answer hinzufügen
                        CType(UrgentOrdersView.CurrentItem, UrgentOrder).Answers.Add(tempObj)
                    End If
                Case NotifyCollectionChangedAction.Remove
                    'Die Answer aus dem Context entfernen
                    Dim ans = _context.AnswerSet.Find(CType(e.OldItems(0), Answer).AnswerId)
                    _context.AnswerSet.Remove(ans)

            End Select
        End Sub

        Private Sub ListView_CurrentChanged(sender As Object, e As EventArgs)
            'Im UrgentOrder-DataGrid wurd das Selektierte Item geändert - wir laden also alle Answers in das AnswersView
            If UrgentOrdersView.CurrentItem IsNot Nothing Then
                If ShownAnswersView IsNot Nothing Then RemoveHandler ShownAnswersView.CollectionChanged, AddressOf AnswersView_Changed 'Wenn das View nicht Nothing dann kann es sein das es einen Handler gibt, diesen mal entfernen
                Me.ShownAnswers = New ObservableCollection(Of Answer)(CType(UrgentOrdersView.CurrentItem, UrgentOrder).Answers.ToList) 'ObservableCollection mit Antworten befüllen
                ShownAnswersView = CollectionViewSource.GetDefaultView(ShownAnswers) 'View mit den Items aus der ObservableCollection erstellen
                AddHandler ShownAnswersView.CollectionChanged, AddressOf AnswersView_Changed 'Handler setzen damit wir es mitbekommen das ein neuer DS hinzugekommen ist oder einer geklöscht wurde
            End If

        End Sub

        Private Sub ListView_Changed(sender As Object, e As NotifyCollectionChangedEventArgs)

            Select Case e.Action
                Case NotifyCollectionChangedAction.Add
                    If GetType(UrgentOrder) Is e.NewItems(0).GetType Then
                        Dim tempObj = CType(e.NewItems(0), UrgentOrder)
                        tempObj.Answers = New List(Of Answer)
                        _context.UrgentOrderSet.Add(tempObj)
                    End If
                Case NotifyCollectionChangedAction.Remove
                    Dim order = _context.UrgentOrderSet.Find(CType(e.OldItems(0), UrgentOrder).UrgentOrderId)
                    'Erst alle Antworten löschen, dann erst die Order da CascadeDeleting deaktiviert
                    For i As Integer = order.Answers.Count - 1 To 0 Step -1
                        _context.AnswerSet.Remove(order.Answers(i))
                    Next
                    _context.UrgentOrderSet.Remove(order)

            End Select
        End Sub

        Private _urgentOrderList As List(Of UrgentOrder)
        Public Property UrgentOrderList() As List(Of UrgentOrder)
            Get
                Return _urgentOrderList
            End Get
            Set(ByVal value As List(Of UrgentOrder))
                _urgentOrderList = value
                RaisePropertyChanged()
            End Set
        End Property


        Private _urgentOrdersView As ICollectionView
        Public Property UrgentOrdersView() As ICollectionView
            Get
                Return _urgentOrdersView
            End Get
            Set(ByVal value As ICollectionView)
                _urgentOrdersView = value
                RaisePropertyChanged()
            End Set
        End Property

        Private _shownAnswersView As ICollectionView
        Public Property ShownAnswersView() As ICollectionView
            Get
                Return _shownAnswersView
            End Get
            Set(ByVal value As ICollectionView)
                _shownAnswersView = value
                RaisePropertyChanged()
            End Set
        End Property

        Private _shownAnswers As ObservableCollection(Of Answer)
        Public Property ShownAnswers() As ObservableCollection(Of Answer)
            Get
                Return _shownAnswers
            End Get
            Set(ByVal value As ObservableCollection(Of Answer))
                _shownAnswers = value
                RaisePropertyChanged()
            End Set
        End Property


        Private _saveCommand As ICommand
        Public Property SaveCommand() As ICommand
            Get
                If _saveCommand Is Nothing Then _saveCommand = New RelayCommand(AddressOf SaveCommand_Execute, AddressOf SaveCommand_CanExecute)
                Return _saveCommand
            End Get
            Set(ByVal value As ICommand)
                _saveCommand = value
                RaisePropertyChanged()
            End Set
        End Property

        Private Sub SaveCommand_Execute(obj As Object)
            MessageBox.Show(_context.SaveChanges & " Änderungen gespeichert")
        End Sub

        Private Function SaveCommand_CanExecute(obj As Object) As Boolean
            If _context Is Nothing Then Return False
            _context.ChangeTracker.DetectChanges()
            Return _context.ChangeTracker.HasChanges()
        End Function

        Protected Overrides Sub Dispose(disposing As Boolean)
            MyBase.Dispose(disposing)

            _context.Dispose()
        End Sub

    End Class
End Namespace